#!/usr/bin/env cdat
# -*- coding: iso-8859-15 -*-

import os, sys, getopt, time
import cdms, cdutil, MV
from cdms.axis import TransientAxis
from cdms.coord import TransientVirtualAxis

from dataproc.utils import cdat

import string

from Numeric import ones

import fonctions_orchidee

import vcs

def usage():
    """convert a restart file in CF convention."""
    
    man = """
Usage : creation of a CF_'restart_file'.nc
    prompt> CF_restart.py restart_file.nc
    
    exemples : CF_restart.py sechiba_rest_out.nc
    """
    print man

options = [
    "help",
    "version" ]

check_test = 0


# Extract arguments from the command line
try:
    myopts, myargs = getopt.getopt(sys.argv[1:], 'Vh', longopts = options)
except getopt.GetoptError:
    print 'getopt error'
    usage()
    sys.exit(1)

if (len(myargs) < 1):
    print 'I need at least 1 file.'
    usage()
    sys.exit(1)
else:
    fich = myargs[0]
    if (os.path.exists(fich)==False):
        print 'file ',fich,' doesn''t exist !' 
        sys.exit(1)

# Affichage des variables : fixe la limite max
MV.set_print_limit(320)

# travail sur le fichier d'entre
f=cdms.open(fich)
listev = f.listvariables()
print "Liste des variables de "+fich,listev


# Write result file
file_out = "CF_"+fich
print '\nWriting to file : ', file_out
fout = cdms.open(file_out, 'w')

# recuperation des axes :
# lon et lat
VarLon=f('nav_lon')
VarLat=f('nav_lat')
axisLon = VarLon.getAxis(0)
axisLat = VarLon.getAxis(1)

# tailles des axes 
jjm = len(axisLon)
iim = len(axisLat)
print "Tailles (iim,jjm) = ",iim,jjm

lon=VarLon[0,:]
lat=VarLat[:,0]

att_lon = cdat.getAttributes(VarLon)
att_lon['name']='lon'
ax_lon = TransientAxis(lon, bounds=None, id='lon', attributes=att_lon, copy=1)

att_lat = cdat.getAttributes(VarLat)
att_lat['name']='lat'
ax_lat = TransientAxis(lat, bounds=None, id='lat', attributes=att_lat, copy=1)
if (check_test): print "Axes lon, lat : \n", ax_lon,"\n", ax_lat

level=f('nav_lev')
att_lev = cdat.getAttributes(level)
ax_lev = TransientAxis(level, bounds=None, id='lev', attributes=att_lev, copy=1)

# Boucle sur les variables 
for var in listev:
    print "*=*==*=*=*=*=*=*=*=*=*=*=*=*=*=**==*"
    print "Variable : ",var
    var0=f(var)
    
    copyattributes = cdat.getAttributes(var0)
    if (check_test): print var,' ; attributs : ',copyattributes
    v0shape = var0.shape
    if (check_test): print "Tailles de "+var+" : ",v0shape
    if (check_test): print "(min / max) de ",var," = ", cdms.MV.minimum(var0),cdms.MV.maximum(var0)


    axes = var0.getAxisList()
    nl = len(axes)

        
    if (check_test): print "Liste des axes de la variable "+var+" : "
    if (check_test): print axes

    listeaxes=[]
    for ax in axes:
        if (check_test): print ax
        if (check_test): print "Attributes de ax : ",ax.attributes
        if  (ax.id == 'x'):
            listeaxes.append(ax_lon)
        elif (ax.id == 'y'):
            listeaxes.append(ax_lat)
        elif (ax.id == 'z' or ax.id == 'lev' or ax.id == 'level'):
            listeaxes.append(ax_lev)
        else:
            listeaxes.append(ax)

    if (check_test): print "liste des axes : ",listeaxes
            
    wvar = cdms.createVariable(var0,
                               axes = listeaxes)
    fout.write(wvar)
    fout.sync()
    print '..... fin du traitement de ',var


# #     div=MV.where(var0_mask<1e-15,1,var0_mask)
        
f.close()
fout.close()
print '...............finished writing.'
# Plus  faire !
# fonctions_orchidee.filtre_dump(file_out)
print '...............finished rewriting.'

